/*
 * Copyright (c) 2007 The Akuma Project
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * $Id: action.c 133 2007-05-29 14:32:04Z asmodai $
 */
/*
    		 Crown Copyright (c) 1997

    This TenDRA(r) Computer Program is subject to Copyright
    owned by the United Kingdom Secretary of State for Defence
    acting through the Defence Evaluation and Research Agency
    (DERA).  It is made available to Recipients with a
    royalty-free licence for its use, reproduction, transfer
    to other parties and amendment for any purpose not excluding
    product development provided that any such use et cetera
    shall be deemed to be acceptance of the following conditions:-

        (1) Its Recipients shall ensure that this Notice is
        reproduced upon any copies or amended versions of it;

        (2) Any amended version of it shall be clearly marked to
        show both the nature of and the organisation responsible
        for the relevant amendment or amendments;

        (3) Its onward transfer from a recipient to another
        party shall be deemed to be that party's acceptance of
        these conditions;

        (4) DERA gives no warranty or assurance as to its
        quality or suitability for any purpose and DERA accepts
        no liability whatsoever in relation to any use to which
        it may be put.
*/


/*** action.c --- Action ADT.
 *
 ** Author: Steve Folkes <smf@hermes.mod.uk>
 *
 *** Commentary:
 *
 * This file implements the action manipulation routines.
 *
 *** Change Log:
 * $Log: action.c,v $
 * Revision 1.1.1.1  1998/01/17  15:57:46  release
 * First version to be checked into rolling release.
 *
 * Revision 1.2  1994/12/15  09:57:54  smf
 * Brought into line with OSSG C Coding Standards Document, as per
 * "CR94_178.sid+tld-update".
 *
 * Revision 1.1.1.1  1994/07/25  16:04:29  smf
 * Initial import of SID 1.8 non shared files.
 *
**/

/****************************************************************************/

#include "action.h"
#include "basic.h"
#include "name.h"
#include "rule.h"
#include "type.h"

/*--------------------------------------------------------------------------*/

ActionP
action_create(void)
{
    ActionP action = ALLOCATE(ActionT);

    types_init(action_param(action));
    types_init(action_result(action));
    action->code = NIL(GenericP);
    return(action);
}

#ifdef FS_FAST
#undef action_param
#endif /* defined (FS_FAST) */
TypeTupleP
action_param(ActionP action)
{
    return(&(action->param));
}
#ifdef FS_FAST
#define action_param(a)	(&((a)->param))
#endif /* defined (FS_FAST) */

#ifdef FS_FAST
#undef action_result
#endif /* defined (FS_FAST) */
TypeTupleP
action_result(ActionP action)
{
    return(&(action->result));
}
#ifdef FS_FAST
#define action_result(a)	(&((a)->result))
#endif /* defined (FS_FAST) */

#ifdef FS_FAST
#undef action_get_code
#endif /* defined (FS_FAST) */
GenericP
action_get_code(ActionP action)
{
    return(action->code);
}
#ifdef FS_FAST
#define action_get_code(a)	((a)->code)
#endif /* defined (FS_FAST) */

#ifdef FS_FAST
#undef action_set_code
#endif /* defined (FS_FAST) */
void
action_set_code(ActionP action, GenericP code)
{
    action->code = code;
}
#ifdef FS_FAST
#define action_set_code(a, c)	((a)->code = (c))
#endif /* defined (FS_FAST) */

void
action_iter_for_table(ActionP action, BoolT full,
		      void(*proc)(EntryP, GenericP), GenericP closure)
{
    if (full) {
	types_iter_for_table(action_param(action), proc, closure);
	types_iter_for_table(action_result(action), proc, closure);
    }
}

/*
 * Local variables(smf):
 * eval: (include::add-path-entry "../os-interface" "../library")
 * eval: (include::add-path-entry "../generated")
 * end:
**/
