/*
 * Copyright (c) 2007 The Akuma Project
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * $Id: entry-list.h 133 2007-05-29 14:32:04Z asmodai $
 */
/*
    		 Crown Copyright (c) 1997

    This TenDRA(r) Computer Program is subject to Copyright
    owned by the United Kingdom Secretary of State for Defence
    acting through the Defence Evaluation and Research Agency
    (DERA).  It is made available to Recipients with a
    royalty-free licence for its use, reproduction, transfer
    to other parties and amendment for any purpose not excluding
    product development provided that any such use et cetera
    shall be deemed to be acceptance of the following conditions:-

        (1) Its Recipients shall ensure that this Notice is
        reproduced upon any copies or amended versions of it;

        (2) Any amended version of it shall be clearly marked to
        show both the nature of and the organisation responsible
        for the relevant amendment or amendments;

        (3) Its onward transfer from a recipient to another
        party shall be deemed to be that party's acceptance of
        these conditions;

        (4) DERA gives no warranty or assurance as to its
        quality or suitability for any purpose and DERA accepts
        no liability whatsoever in relation to any use to which
        it may be put.
*/


/*** entry-list.h --- Identifier table entry list ADT.
 *
 ** Author: Steve Folkes <smf@hermes.mod.uk>
 *
 *** Commentary:
 *
 * See the file "entry-list.h" for more information.
 *
 *** Change Log:
 * $Log: entry-list.h,v $
 * Revision 1.1.1.1  1998/01/17  15:57:46  release
 * First version to be checked into rolling release.
 *
 * Revision 1.2  1994/12/15  09:58:06  smf
 * Brought into line with OSSG C Coding Standards Document, as per
 * "CR94_178.sid+tld-update".
 *
 * Revision 1.1.1.1  1994/07/25  16:04:33  smf
 * Initial import of SID 1.8 non shared files.
 *
**/

/****************************************************************************/

#ifndef H_ENTRY_LIST
#define H_ENTRY_LIST

#include "os-interface.h"
#include "dalloc.h"
#include "entry.h"
#include "ostream.h"

/*--------------------------------------------------------------------------*/

typedef struct EntryListEntryT {
    struct EntryListEntryT     *next;
    EntryP			entry;
} EntryListEntryT, *EntryListEntryP;

typedef struct EntryListT {
    EntryListEntryP		head;
    EntryListEntryP	       *tail;
} EntryListT, *EntryListP;

typedef struct SaveListT {
    struct EntryListEntryT    **last_ref;
} SaveListT, *SaveListP;

/*--------------------------------------------------------------------------*/

extern void	entry_list_init(EntryListP);
extern void	entry_list_copy(EntryListP, EntryListP);
extern void	entry_list_add(EntryListP, EntryP);
extern void	entry_list_add_if_missing(EntryListP, EntryP);
extern BoolT	entry_list_contains(EntryListP, EntryP);
extern BoolT	entry_list_includes(EntryListP, EntryListP);
extern void	entry_list_intersection(EntryListP, EntryListP, EntryListP);
extern void	entry_list_unlink_used(EntryListP, EntryListP);
extern void	entry_list_append(EntryListP, EntryListP);
extern BoolT	entry_list_is_empty(EntryListP);
extern void	entry_list_save_state(EntryListP, SaveListP);
extern void	entry_list_restore_state(EntryListP, SaveListP);
extern void	entry_list_iter(EntryListP, void(*)(EntryP, GenericP),
				GenericP);
extern void	entry_list_iter_table(EntryListP, BoolT,
				      void(*)(EntryP, GenericP), GenericP);
extern void	entry_list_destroy(EntryListP);

extern void	write_entry_list(OStreamP, EntryListP);

/*--------------------------------------------------------------------------*/

#ifdef FS_FAST
#define entry_list_is_empty(e)	((e)->head == NIL(EntryListEntryP))
#endif /* defined (FS_FAST) */

#endif /* !defined (H_ENTRY_LIST) */

/*
 * Local variables(smf):
 * eval: (include::add-path-entry "../os-interface" "../library")
 * eval: (include::add-path-entry "../generated")
 * end:
**/
