/*
 * Copyright (c) 2007 The Akuma Project
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * $Id: non-local.c 133 2007-05-29 14:32:04Z asmodai $
 */
/*
    		 Crown Copyright (c) 1997

    This TenDRA(r) Computer Program is subject to Copyright
    owned by the United Kingdom Secretary of State for Defence
    acting through the Defence Evaluation and Research Agency
    (DERA).  It is made available to Recipients with a
    royalty-free licence for its use, reproduction, transfer
    to other parties and amendment for any purpose not excluding
    product development provided that any such use et cetera
    shall be deemed to be acceptance of the following conditions:-

        (1) Its Recipients shall ensure that this Notice is
        reproduced upon any copies or amended versions of it;

        (2) Any amended version of it shall be clearly marked to
        show both the nature of and the organisation responsible
        for the relevant amendment or amendments;

        (3) Its onward transfer from a recipient to another
        party shall be deemed to be that party's acceptance of
        these conditions;

        (4) DERA gives no warranty or assurance as to its
        quality or suitability for any purpose and DERA accepts
        no liability whatsoever in relation to any use to which
        it may be put.
*/


/**** non-local.c --- Non local name ADT.
 *
 ** Author: Steve Folkes <smf@hermes.mod.uk>
 *
 **** Commentary:
 *
 * This file implements the non local name list manipulation routines.
 *
 **** Change Log:
 * $Log: non-local.c,v $
 * Revision 1.1.1.1  1998/01/17  15:57:46  release
 * First version to be checked into rolling release.
 *
 * Revision 1.2  1994/12/15  09:58:27  smf
 * Brought into line with OSSG C Coding Standards Document, as per
 * "CR94_178.sid+tld-update".
 *
 * Revision 1.1.1.1  1994/07/25  16:04:36  smf
 * Initial import of SID 1.8 non shared files.
 *
**/

/****************************************************************************/

#include "non-local.h"

/*--------------------------------------------------------------------------*/

void
non_local_list_init(NonLocalListP non_locals)
{
    non_locals->head = NIL(NonLocalEntryP);
    non_locals->tail = &(non_locals->head);
}

NonLocalEntryP
non_local_list_add(NonLocalListP non_locals, EntryP name, EntryP type)
{
    NonLocalEntryP entry = ALLOCATE(NonLocalEntryT);

    entry->next         = NIL(NonLocalEntryP);
    entry->name         = name;
    entry->type         = type;
    entry->initialiser  = NIL(EntryP);
    *(non_locals->tail) = entry;
    non_locals->tail    = &(entry->next);
    return(entry);
}

BoolT
non_local_list_is_empty(NonLocalListP non_locals)
{
    return(non_locals->head == NIL(NonLocalEntryP));
}

void
non_local_list_iter_for_table(NonLocalListP non_locals,
			      void (*proc)(EntryP, GenericP),
			      GenericP closure)
{
    NonLocalEntryP non_local;

    for (non_local = non_locals->head; non_local;
	 non_local = non_local->next) {
	entry_iter(non_local->type, TRUE, proc, closure);
	if (non_local->initialiser) {
	    entry_iter(non_local->initialiser, TRUE, proc, closure);
	}
    }
}

void
non_local_list_destroy(NonLocalListP non_locals)
{
    NonLocalEntryP entry = non_locals->head;

    while (entry) {
	NonLocalEntryP tmp = entry->next;

	DEALLOCATE(entry);
	entry = tmp;
    }
}

void
write_non_locals(OStreamP ostream, NonLocalListP non_locals)
{
    NonLocalEntryP non_local;

    for (non_local = non_locals->head; non_local;
	 non_local = non_local->next) {
	ASSERT(non_local->type);
	ASSERT(non_local->name);
	write_tab(ostream);
	write_key(ostream, entry_key(non_local->name));
	write_cstring(ostream, ": ");
	write_key(ostream, entry_key(non_local->type));
	if (non_local->initialiser) {
	    write_cstring(ostream, " = <");
	    write_key(ostream, entry_key(non_local->initialiser));
	    write_char(ostream, '>');
	}
	write_char(ostream, ';');
	write_newline(ostream);
    }
}

void
non_local_entry_set_initialiser(NonLocalEntryP non_local, EntryP init)
{
    non_local->initialiser = init;
}

EntryP
non_local_entry_get_initialiser(NonLocalEntryP non_local)
{
    return(non_local->initialiser);
}

EntryP
non_local_entry_get_name(NonLocalEntryP non_local)
{
    return(non_local->name);
}

EntryP
non_local_entry_get_type(NonLocalEntryP non_local)
{
    return(non_local->type);
}

/*
 * Local variables(smf):
 * eval: (include::add-path-entry "../os-interface" "../library")
 * eval: (include::add-path-entry "../generated")
 * end:
**/
